package gov.usgs.earthquake.nshmp.calc;

import gov.usgs.earthquake.nshmp.gmm.GmmInput;
import gov.usgs.earthquake.nshmp.gmm.GroundMotionModel;
import gov.usgs.earthquake.nshmp.model.Rupture;

/**
 * A {@link GroundMotionModel} input that carries {@link Rupture} rate
 * information along with it
 *
 * @author U.S. Geological Survey
 * @see GmmInput
 */
public final class HazardInput extends GmmInput {

  final double rate;

  public HazardInput(
      double rate,
      double Mw, double rJB, double rRup, double rX,
      double dip, double width, double zTor, double zHyp, double rake,
      double vs30, boolean vsInf, double z1p0, double z2p5) {

    super(GmmInput.builder()
        .dip(dip)
        .mag(Mw)
        .rake(rake)
        .rJB(rJB)
        .rRup(rRup)
        .rX(rX)
        .vs30(vs30)
        .vsInf(vsInf)
        .width(width)
        .z1p0(z1p0)
        .z2p5(z2p5)
        .zHyp(zHyp)
        .zTor(zTor));

    this.rate = rate;
  }

  @Override
  public String toString() {
    return getClass().getSimpleName() + " [rate=" + String.format("%.4g", rate) + " " +
        super.toString() + "]";
  }

}
