package gov.usgs.earthquake.nshmp.fault.surface;

import gov.usgs.earthquake.nshmp.geo.Location;
import gov.usgs.earthquake.nshmp.model.Distance;

/**
 * A parameterization of an earthquake rupture surface.
 *
 * @author U.S. Geological Survey
 */
public interface RuptureSurface {

  /**
   * The average strike of this surface in degrees [0°..360°).
   */
  double strike();

  /**
   * The average dip of this surface in degrees [0°..90°].
   */
  double dip();

  /**
   * The average dip direction of this surface in degrees [0°..360°).
   */
  double dipDirection();

  /**
   * The average length of the upper edge or trace of this surface in km.
   */
  double length();

  /**
   * The average down-dip width of this surface in km.
   */
  double width();

  /**
   * The area of this surface in km².
   */
  double area();

  /**
   * The average depth to the top of this surface in km (always positive)
   */
  double depth();

  /**
   * The centroid of this surface.
   */
  Location centroid();

  /**
   * Returns the distance metrics commonly required by PSHA ground motion models
   * (GMMs): rJB, rRup, and rX.
   *
   * @param location to compute distances to
   * @return a distance metric wrapper object
   * @see Distance
   */
  Distance distanceTo(Location location);

}
