package gov.usgs.earthquake.nshmp.gmm;

/**
 * Ground motion model (Gmm) result container.
 *
 * @author U.S. Geological Survey
 */
public interface GroundMotion {

  /**
   * Returns the mean (natural log of the median) ground motion.
   * @return the mean
   */
  public double mean();

  /**
   * Returns the standard deviation in natural log units.
   * @return the standard deviation
   */
  public double sigma();

  /**
   * Create a new ground motion container.
   *
   * @param mean ground motion (in natural log units)
   * @param sigma aleatory uncertainty
   * @return a new scalar ground motion container.
   */
  public static GroundMotion create(double mean, double sigma) {
    return new GroundMotion() {

      @Override
      public double mean() {
        return mean;
      }

      @Override
      public double sigma() {
        return sigma;
      }

      @Override
      public String toString() {
        return new StringBuilder(getClass().getSimpleName())
            .append(" [Median: ")
            .append(String.format("%.6f", Math.exp(mean)))
            .append(" Sigma: ")
            .append(String.format("%.6f", sigma))
            .append("]")
            .toString();
      }
    };
  }
}
