package gov.usgs.earthquake.nshmp.gmm;

/**
 * NEHRP site class identifier.
 *
 * <p>Over time, the equivalent Vs30 values for each site class have changed
 * slightly and are documented with each NSHM.
 *
 * @author U.S. Geological Survey
 */
public enum NehrpSiteClass {

  /** Site class A, very hard rock. */
  A("very hard rock"),

  /** Site class A/B boundary, hard rock. */
  AB("hard rock"),

  /** Site class B, medium hard rock. */
  B("medium hard rock"),

  /** Site class B/C boundary, soft rock. */
  BC("soft rock"),

  /** Site class C, very dense soil or hard clay. */
  C("very dense soil or hard clay"),

  /** Site class C/D boundary, dense sand or very stiff clay. */
  CD("dense sand or very stiff clay"),

  /** Site class D, medium dense sand or stiff clay. */
  D("medium dense sand or stiff clay"),

  /** Site class D/E boundary, loose sand or medium stiff clay. */
  DE("loose sand or medium stiff clay"),

  /** Site class E, very loose sand or soft clay. */
  E("very loose sand or soft clay");

  private final String condition;

  private NehrpSiteClass(String condition) {
    this.condition = condition;
  }

  public String condition() {
    return condition;
  }
}
