package gov.usgs.earthquake.nshmp.model;

import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.base.Preconditions.checkState;

import com.google.common.collect.Range;

import gov.usgs.earthquake.nshmp.data.DoubleData;

/*
 * Ground motion model (GMM) configuration.
 *
 * @author U.S. Geological Survey
 */
class GmmConfig {

  static final Range<Double> MAX_DIST_RANGE = Range.closed(50.0, 1000.0);
  static final String MAX_DISTANCE_KEY = "max-distance";

  final double maxDistance;

  private GmmConfig(Builder builder) {
    this.maxDistance = builder.maxDistance;
  }

  static Builder builder() {
    return new Builder();
  }

  /* Single use builder. */
  static class Builder {

    private Double maxDistance;
    private boolean built = false;

    Builder maximumDistance(double rMax) {
      DoubleData.checkInRange(MAX_DIST_RANGE, MAX_DISTANCE_KEY, rMax);
      this.maxDistance = rMax;
      return this;
    }

    GmmConfig build() {
      checkState(!built, "GmmConfig.Builder has already been used");
      checkNotNull(maxDistance, "%s required", MAX_DISTANCE_KEY);
      built = true;
      return new GmmConfig(this);
    }
  }
}
