package gov.usgs.earthquake.nshmp.tree;

import static com.google.common.base.Preconditions.checkNotNull;
import static gov.usgs.earthquake.nshmp.data.DoubleData.checkWeight;

/**
 * Branch with id and value backed by an enum.
 *
 * @author U.S. Geological Survey
 */
class EnumValueBranch<E extends Enum<E>> implements Branch<E> {

  private final E id;
  private final double weight;

  EnumValueBranch(E id, double weight) {
    this.id = checkNotNull(id);
    this.weight = checkWeight(weight);
  }

  @Override
  public String id() {
    return id.name();
  }

  @Override
  public E value() {
    return id;
  }

  @Override
  public double weight() {
    return weight;
  }

  @Override
  public String toString() {
    return RegularBranch.toString(id(), weight);
  }
}
