package gov.usgs.earthquake.nshmp.tree;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;
import static gov.usgs.earthquake.nshmp.data.DoubleData.checkWeight;

import gov.usgs.earthquake.nshmp.Maths;

/**
 * Basic logic tree branch.
 *
 * @author U.S. Geological Survey
 */
class RegularBranch<T> implements Branch<T> {

  private final String id;
  private final T value;
  private final double weight;

  RegularBranch(String id, T value, double weight) {
    this.id = checkId(id);
    this.value = checkNotNull(value);
    this.weight = checkWeight(Maths.round(weight, RegularTree.WEIGHT_SCALE));
  }

  @Override
  public String id() {
    return id;
  }

  @Override
  public T value() {
    return value;
  }

  @Override
  public double weight() {
    return weight;
  }

  @Override
  public String toString() {
    return toString(id, weight);
  }

  static String checkId(String id) {
    checkArgument(!checkNotNull(id).trim().isEmpty(), "Empty branch id");
    return id;
  }

  static String toString(String id, double weight) {
    return id + " [" + weight + "]";
  }
}
