package gov.usgs.earthquake.nshmp.calc;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

import org.junit.jupiter.api.Test;

import gov.usgs.earthquake.nshmp.model.SiteData;

class SitesTests {

  static final Path RESOURCE_PATH = Paths.get("src/test/resources/calc");
  static final Path CSV_PATH = RESOURCE_PATH.resolve("sites.csv");
  static final Path CSV_NO_PROPS_PATH = RESOURCE_PATH.resolve("sites-no-props.csv");
  static final Path GEOJSON_POINTS_PATH = RESOURCE_PATH.resolve("sites-points.geojson");
  static final Path GEOJSON_POLYGON_PATH = RESOURCE_PATH.resolve("sites-polygon.geojson");

  @Test
  final void testCsv() throws IOException {
    CalcConfig config = CalcConfig.defaults().build();

    List<Site> sites = Sites.fromCsv(CSV_PATH, config, SiteData.EMPTY);
    assertEquals(6, sites.size());

    List<Site> sitesNoProps = Sites.fromCsv(CSV_NO_PROPS_PATH, config, SiteData.EMPTY);
    assertEquals(6, sitesNoProps.size());

    // toString
    assertEquals(TO_STRING, Sites.toString(sites));
  }

  @Test
  final void testGeoJson() throws IOException {
    CalcConfig config = CalcConfig.defaults().build();

    List<Site> sitesPoints = Sites.fromGeoJson(GEOJSON_POINTS_PATH, config, SiteData.EMPTY);
    assertEquals(3, sitesPoints.size());

    List<Site> sitesPolygon = Sites.fromGeoJson(GEOJSON_POLYGON_PATH, config, SiteData.EMPTY);
    assertEquals(11654, sitesPolygon.size());
  }

  static final String TO_STRING =
      "Sites [size=6]\n" +
          "               Hilo HI                     -155.060 19.700 Vs30=760.0 inferred  Z1.0=NaN Z2.5=NaN\n" +
          "               Salt Lake City UT           -111.900 40.750 Vs30=530.0 inferred  Z1.0=0.3 Z2.5=3.0\n" +
          "               San Jose CA                 -121.900 37.350 Vs30=350.0 measured  Z1.0=NaN Z2.5=NaN\n" +
          "               Seattle WA                  -122.300 47.600 Vs30=760.0 inferred  Z1.0=1.2 Z2.5=6.7\n" +
          "               Memphis TN                  -90.050 35.150 Vs30=3000.0 inferred  Z1.0=NaN Z2.5=NaN\n" +
          "               ... and 1 more ...";

}
