package gov.usgs.earthquake.nshmp.gmm;

import static gov.usgs.earthquake.nshmp.gmm.Gmm.AB_03_CASCADIA_INTERFACE;
import static gov.usgs.earthquake.nshmp.gmm.Gmm.AB_03_GLOBAL_INTERFACE;
import static gov.usgs.earthquake.nshmp.gmm.Gmm.AM_09_INTERFACE;
import static gov.usgs.earthquake.nshmp.gmm.Gmm.BCHYDRO_12_INTERFACE;
import static gov.usgs.earthquake.nshmp.gmm.Gmm.YOUNGS_97_INTERFACE;
import static gov.usgs.earthquake.nshmp.gmm.Gmm.ZHAO_06_INTERFACE;
import static gov.usgs.earthquake.nshmp.gmm.Imt.PGA;
import static gov.usgs.earthquake.nshmp.gmm.Imt.SA0P2;
import static gov.usgs.earthquake.nshmp.gmm.Imt.SA1P0;
import static gov.usgs.earthquake.nshmp.gmm.Imt.SA3P0;

import java.io.IOException;
import java.util.EnumSet;
import java.util.Set;
import java.util.stream.Stream;

import org.junit.jupiter.api.extension.ExtensionContext;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.ArgumentsProvider;
import org.junit.jupiter.params.provider.ArgumentsSource;

class SubInterface implements ArgumentsProvider {

  private static String GMM_INPUTS = "/gmm/interface-inputs.csv";
  private static String GMM_RESULTS = "/gmm/interface-results.csv";

  @ParameterizedTest(name = "({index}) {0} {2} {1}")
  @ArgumentsSource(SubInterface.class)
  void test(int index, Gmm gmm, Imt imt, double exMedian, double exSigma, String inputs) {
    GmmTest.test(index, gmm, imt, exMedian, exSigma, inputs);
  }

  @Override
  public Stream<? extends Arguments> provideArguments(ExtensionContext context) throws Exception {
    return GmmTest.loadResults(GMM_RESULTS, GMM_INPUTS);
  }

  /* Result generation sets */
  private static Set<Gmm> gmms = EnumSet.of(
      AB_03_GLOBAL_INTERFACE,
      AB_03_CASCADIA_INTERFACE,
      AM_09_INTERFACE,
      BCHYDRO_12_INTERFACE,
      YOUNGS_97_INTERFACE,
      ZHAO_06_INTERFACE);

  private static Set<Imt> imts = EnumSet.of(
      PGA,
      SA0P2,
      SA1P0,
      SA3P0);

  public static void main(String[] args) throws IOException {
    GmmTest.generateResults(gmms, imts, GMM_INPUTS, GMM_RESULTS);
  }

}
